import { __extends, __assign } from 'tslib';
import { Record } from 'immutable';
import isPlainObject from 'is-plain-obj';
import tinycolor from 'tinycolor2';
import { Mark, Data } from 'slate';
import { hasMark, removeCollapsedMark, removeExpandedMark } from '@vericus/slate-kit-plugins-utils';
import Renderer from '@vericus/slate-kit-highlight-text-renderer';

var defaultOptions = {
    marks: undefined,
    type: undefined,
    styles: undefined,
    data: undefined,
    defaultColor: undefined,
    alpha: 1,
    externalRenderer: false
};
var Options = /** @class */ (function (_super) {
    __extends(Options, _super);
    function Options() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Options.create = function (attrs) {
        if (attrs === void 0) { attrs = {}; }
        if (Options.isOptions(attrs))
            return attrs;
        if (isPlainObject(attrs))
            return Options.fromJSON(attrs);
        throw new Error("`Options.create` only accepts objects, strings or options");
    };
    Options.fromJSON = function (object) {
        var _a;
        if (Options.isOptions(object))
            return object;
        var type = object.type, data = object.data, styles = object.styles, defaultColor = object.defaultColor, _b = object.alpha, alpha = _b === void 0 ? 1 : _b;
        if (!type || !data || !styles || !defaultColor || !Array.isArray(styles)) {
            throw new Error("Highlight text require type, data and style option");
        }
        if (alpha < 0 || alpha > 1) {
            throw new Error("Alpha value can only go from 0 to 1");
        }
        return new Options({
            marks: (_a = {},
                _a[type] = type,
                _a),
            type: type,
            data: data,
            styles: styles,
            alpha: alpha,
            defaultColor: defaultColor
        });
    };
    Options.isOptions = function (args) {
        return !!(args instanceof Record &&
            ["marks", "type", "data", "styles", "defaultColor"].every(function (key) {
                return args.has(key);
            }));
    };
    return Options;
}(Record(defaultOptions, "highlight text option")));

var defaultStyle = {
    textDecoration: "inherit",
    textDecorationColor: "inherit"
};
function createProps(opts) {
    var type = opts.type, dataField = opts.data, styleFields = opts.styles, alpha = opts.alpha;
    return {
        getProps: function (props) {
            if (!props.mark || props.mark.type !== type)
                return props;
            var children = props.children, attributes = props.attributes, data = props.mark.data;
            var style = (attributes || { style: {} }).style;
            var color = tinycolor(data.get(dataField));
            style = __assign({}, defaultStyle, style, { textDecoration: "inherit", decorationStyle: "initial" }, styleFields.reduce(function (acc, styleField) {
                var _a;
                return (__assign({}, acc, (_a = {}, _a[styleField] = color.setAlpha(alpha).toRgbString(), _a)));
            }, {}));
            return __assign({}, props, { attributes: __assign({}, props.attributes, { style: style }) });
        }
    };
}

function createChanges(opt) {
    var defaultColor = opt.defaultColor, data = opt.data, type = opt.type;
    return {
        changeColor: function (change, color) {
            var _a, _b;
            var value = change.value;
            var selection = value.selection, document = value.document;
            if (hasMark(value, type)) {
                if (tinycolor(color).toName() === defaultColor) {
                    if (selection.isCollapsed) {
                        removeCollapsedMark(value, change, type);
                    }
                    else {
                        removeExpandedMark(document, selection, change, type);
                    }
                }
                else {
                    var newMark = new Mark({
                        type: type,
                        data: Data.create((_a = {},
                            _a[data] = color,
                            _a))
                    });
                    if (selection.isCollapsed) {
                        removeCollapsedMark(value, change, type);
                        change.focus();
                        change.addMark(newMark);
                    }
                    else {
                        removeExpandedMark(document, selection, change, type);
                        change.addMarkAtRange(selection, newMark);
                    }
                }
            }
            else if (tinycolor(color).toName() !== defaultColor) {
                var mark = new Mark({
                    type: type,
                    data: Data.create((_b = {},
                        _b[data] = color,
                        _b))
                });
                if (selection.isCollapsed) {
                    change.addMark(mark);
                }
                else {
                    removeExpandedMark(document, selection, change, type);
                    change.addMarkAtRange(selection, mark);
                }
            }
            change.focus();
            return change;
        }
    };
}

function createUtils(opts) {
    var defaultColor = opts.defaultColor, data = opts.data, type = opts.type;
    return {
        currentColor: function (value) {
            if (value.selection && value.selection.isFocused && value.activeMarks) {
                var activeColorMarks = value.activeMarks
                    .filter(function (mark) { return !!(mark && mark.type === type); })
                    .reduce(function (colors, mark) {
                    if (mark && mark.data.get(data)) {
                        return colors.concat([mark.data.get(data)]);
                    }
                    return colors;
                }, []);
                var uniqueColorMarks = Array.from(new Set(activeColorMarks)).slice();
                if (uniqueColorMarks.length !== 0) {
                    return uniqueColorMarks[0];
                }
            }
            return defaultColor;
        }
    };
}

function createPlugin(pluginOptions, pluginsWrapper) {
    var options = Options.create(pluginOptions);
    var type = options.type;
    var changes = createChanges(options);
    var utils = createUtils(options);
    var props = createProps(options);
    var plugin = {
        options: options,
        changes: changes,
        utils: utils,
        props: props
    };
    if (!options.externalRenderer) {
        var renderers = Renderer(type).renderers;
        plugin.renderers = renderers;
    }
    return plugin;
}

export default createPlugin;
//# sourceMappingURL=slate-kit-highlight-text.es.js.map
